%%% -*- indent-tabs-modeL nil -*-
%%%
%%% Author:
%%%   Andreas Franke <afranke@ags.uni-sb.de>
%%%
%%% Copyright:
%%%   Andreas Franke, 2000
%%%
%%% Last change:
%%%   $Date: 2001/09/23 13:37:33 $ by $Author: afranke $
%%%   $Revision: 1.3 $
%%%
%%% This file is part of MathWeb
%%%    http://www.mathweb.org/mathweb
%%%
%%% See the file "LICENSE" or
%%%    http://www.mathweb.org/mathweb/LICENSE.html
%%% for information on usage and redistribution
%%% of this file, and for a DISCLAIMER OF ALL
%%% WARRANTIES.
%%%

%%
%% a simple XMLRPC client
%%

functor
   
import
   Error(registerFormatter)
   
   HttpClient
   at 'x-ozlib://mathweb/httpd/HttpClient.ozf'

   XmlRpcWriter
   at 'x-ozlib://mathweb/xmlrpc/xmlrpc.printer.ozf'
   
   XmlParser
   at 'x-ozlib://mathweb/xmlparser/XmlParser.ozf'
   PModule
   at 'x-ozlib://mathweb/xmlrpc/xmlrpc.parser.ozf'
   SModule
   at 'x-ozlib://mathweb/xmlrpc/xmlrpc.sem.ozf' 

export
   'class': XmlRpcClient
   execute: Execute % proc {$ URL MethodName Params ?Result}
   
define

   ParserModule = {XmlParser.link PModule SModule}
   %%{XmlParser.setDebug true}
   
   %% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%% xmlrpc errors %%%%%%%%%%%%%%%%%%%%%%%%%
   {Error.registerFormatter xmlrpc
    fun {$ E}
       T = 'xmlrpc error'
    in 
       case E of xmlrpc(R=http(StatusCode ReasonPhrase...)...) then
          error(kind:T msg: 'HTTP Error'
                items: [hint(l:'Status Code'   m:StatusCode)
                        hint(l:'Reason Phrase' m:ReasonPhrase)
                        unit
                        hint(l:'Response Content'
                             m:"\n"#{CondSelect R content nil})
                       ])
       [] xmlrpc(parser(...)...) then
          error(kind:T msg: 'Can\'t parse method response'
                items: nil)
       [] xmlrpc(fault(FaultCode FaultString...)...) then 
          error(kind:T msg: 'Remote Error'
                items: [hint(l:'Fault Code'   m:FaultCode)
                        hint(l:'Fault String' m:FaultString)])
       else
          error(kind:T items:[line(oz(E))])
       end
    end}
   %% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   
   class Client
      from
         HttpClient.'class'
         XmlRpcWriter.'class'
   end

   class Self = XmlRpcClient

      feat
         url
         headers
         
      meth init(URL headers:Headers <= unit)
         self.url = URL
         self.headers = {Adjoin unit('content-type':'text/xml') Headers}
      end
      meth execute(MethodName Params ?Result
                   response: ?Response <= _)
         C = {New Client init}
         {C beginPOST(self.url headers: self.headers)}
         {C 'write_methodCall'(MethodName Params)}
         {C endPOST}
         HTTPVersion StatusCode ReasonPhrase Headers Content
         {C readResponse(?HTTPVersion ?StatusCode ?ReasonPhrase
                         ?Headers ?Content)}
         {C close}
      in
         Response = response(httpVersion:  HTTPVersion
                             statusCode:   StatusCode
                             reasonPhrase: ReasonPhrase
                             headers:      Headers
                             content:      Content)
         if StatusCode\=200 then
            Self, RaiseError(http(StatusCode ReasonPhrase
                                  httpVersion: HTTPVersion
                                  headers:     Headers
                                  content:     Content))
         else
            Val Accept in
            {XmlParser.parseVS Content ParserModule
             'start_methodResponse'(?Val) ?Accept}
            if Accept==false then
               Self, RaiseError(parser)
            elsecase Val of fault(...) then
               Self, RaiseError(fault(Val.faultCode Val.faultString))
            [] ok(X) then
               Result=X
            end
         end
      end
      meth RaiseError(Err)
         {Exception.raiseError xmlrpc(Err url:self.url)} 
      end
      meth close(...)
         skip
      end
   end
   
   proc {Execute URL MethodName Params ?Result}
      C = {New XmlRpcClient init(URL)} in
      {C execute(MethodName Params ?Result)}
      {C close}
   end
end

