%%%
%%% Author:
%%%   Juergen Zimmer <jzimmer@mathweb.org>
%%%   Andreas Franke <afranke@mathweb.org>
%%%
%%% Copyright:
%%%   Juergen Zimmer, February 2001
%%%
%%% Last change:
%%%   $Date $ by $Author $
%%%   $Revision $
%%%
%%% This file is part of MathWeb
%%%    http://www.mathweb.org/mathweb
%%%
%%% See the file "LICENSE" or
%%%    http://www.mathweb.org/mathweb/LICENSE.html
%%% for information on usage and redistribution
%%% of this file, and for a DISCLAIMER OF ALL
%%% WARRANTIES.
%%%


/**
This is a generic XML-RPC server that handles XML-remote-procedure-calls.
It is fully compatible with the Java XML-RPC package of
   Hannes Wallnoefer (see http://www.xmlrpc.com/).
It handles XMLRPC-requests are of the form:

POST /RPC2 HTTP/1.0
Content-Type: text/xml
Content-Length: 100
<methodCall>
<methodName>Math.add</methodName>
<params>
<param>
<value><i4>41</i4></value>
</param>
</params>
</methodCall>
*/

functor

import
   Error(exceptionToMessage messageToVirtualString)
   Property(put)
   URL(make toVirtualString)
%   Browser(browse: Browse)
   System
   
   Http(server:     HttpServer
        connection: HttpConnection)
   at 'x-ozlib://mathweb/httpd/Http.ozf'
   
   XmlParser
   at 'x-ozlib://mathweb/xmlparser/XmlParser.ozf'
   PModule
   at 'x-ozlib://mathweb/xmlrpc/xmlrpc.parser.ozf'
   SModule
   at 'x-ozlib://mathweb/xmlrpc/xmlrpc.sem.ozf'
   XmlrpcWriter('class':Xmlrpc)
   at 'x-ozlib://mathweb/xmlrpc/xmlrpc.printer.ozf'

export
   server:     XmlRpcServer
   connection: XmlRpcConnection
   
define
   VS2A = VirtualString.toAtom
   VS2S = VirtualString.toString

   fun {ExceptionToVS Exc}
      {Error.messageToVirtualString {Error.exceptionToMessage Exc}} 
   end
   
%   Debug = true
   Debug = false
   
   {Property.put 'errors.width' 100}
   {Property.put 'errors.depth' 100}
   
   ParserModule = {XmlParser.link PModule SModule}

   {XmlParser.setDebug Debug}
   
   %% =======================================
   %%  The connection manager handles the different connections to the PORT
   %% =======================================
   class XmlRpcConnection
      from
         HttpConnection
         Xmlrpc
         
      meth init(...) = Init
         Xmlrpc, Init
      end
      
      meth beginResponseBase
         HttpConnection, beginResponse
         {self tag(base(href:self.server.baseURL)#"\n")}
      end
      
      meth 'GET'(X...) = GET
         U = {URL.make X}
         P = U.path
      in
         case P of [nil] then
            HttpConnection, 'GET'("/index.html")
         else
            HttpConnection, GET
         end
      end
      
      meth 'POST'(X Content...) = POST
         U = {URL.make X}         
         U1 = {Record.subtractList U [device scheme authority query fragment]}
         P = {URL.toVirtualString U1}
      in 
         %%{Browse POST}
         case {Dictionary.condGet self.server.services {VS2A P} unit}
         of unit then
            HttpConnection, errNotFound
         [] CallFun then
            MethodCall = {self ParseRequest(Content $)} in
            try
               Result={CallFun {VS2S MethodCall.methodName} MethodCall.params}
            in
               {self WriteResponse(writeResult(Result))}
            catch Err then
               case Err of fault(FaultCode FaultString) then
                  {self WriteResponse(writeFault(FaultCode FaultString))}
               else
                  {self WriteResponse(writeFault(0 "Some Error occured..."))}
                  %%{ExceptionToVS Err}))}
               end
            end
            {Delay 1000}
            raise close end
         end
      end
      
      meth ParseRequest(Request ?MethodCall)
         Accept in
         {XmlParser.parseVS Request ParserModule
          'start_methodCall'(?MethodCall) ?Accept}
         if Accept==false then
            {self beginResponse(400)}
            {self doctype}
            {self tag(html(head(title("400 Parse Error"))
                           body(h1("Parse Error")
                                "Sorry, your request could not be parsed.")))}
            {self endResponse}
            {Delay 1000}
            raise close end
         end
      end

      meth WriteResponse(Meth)
         {self beginResponse(200 contentType:'text/xml')}
         %%HttpConnection, headerLine('content-encoding' 'us-ascii')
         Xmlrpc, Meth
         {self endResponse}
      end
   end
   
   %% =======================================
   %% the server provides access to the registered services
   %% =======================================

   class XmlRpcServer
      from HttpServer
         
      feat
         services

      meth init(...) = Init
         self.services = {NewDictionary}
         HttpServer, Init
      end
      meth register(Path S)
         Key = {VS2A Path} in
         {Dictionary.put self.services Key S}
      end
      meth unregister(Path)
         Key = {VS2A Path} in
         {Dictionary.put self.services Key unit}
      end
      
   end
   
end
