%%%
%%% Author:
%%%   Andreas Franke <afranke@mathweb.org>
%%%   Juergen Zimmer <jzimmer@mathweb.org>
%%%   
%%% Copyright:
%%%   Juergen Zimmer, 2000-2001
%%%
%%% Last change:
%%%   $Date: 2001/08/31 10:10:33 $ by $Author: jzimmer $
%%%   $Revision: 1.2 $
%%%
%%% This file is part of MathWeb
%%%    http://www.mathweb.org/mathweb
%%%
%%% See the file "LICENSE" or
%%%    http://www.mathweb.org/mathweb/LICENSE.html
%%% for information on usage and redistribution
%%% of this file, and for a DISCLAIMER OF ALL
%%% WARRANTIES.
%%%

%%
%% AppletServer 
%%

functor

import
   OS(tmpnam unlink rand srand)
   URL(make)
   
   File(loadBS)
   at 'x-ozlib://mathweb/share/File.ozf'

   HttpHandler('class':HTTPHandler)
   at 'x-ozlib://mathweb/httpd/HttpHandler.ozf'
   
   %%XmlRpcHandler('class':RpcHandler)
   %%at 'x-ozlib://mathweb/xmlrpc/XmlRpcServer.ozf'
   
   MakeApplet(generic)
   at 'x-ozlib://mathweb/loui/MakeApplet.ozf'
   
export
   createILoui: CreateILouiApplet
   httpHandler: GetAppletHandler
   
define
   VS2BS= VirtualString.toByteString

   %%
   %% registry for temporary files (to be used for the individualized applets) 
   %%
   local
      {OS.srand 0}
      NotFound = {NewName}
      
      class GenericRegistry
         prop
            locking
         feat
            dict
            
         meth init
            self.dict = {NewDictionary}
         end
         %%
         meth generateID(_ $)
            {OS.rand}
         end
         %%
         meth generateNewID(Val $)
            ID = {self generateID(Val $)} in
            if {Dictionary.member self.dict ID}
            then {self generateNewID(Val $)}
            else ID
            end
         end
         %%
         meth put(Val ?ID)
            lock
               ID = {self generateNewID(Val $)}
               {Dictionary.put self.dict ID Val}
               %%{App.show put(Val ID)}
            end
         end
         %%
         meth get(ID $)
            lock
               case {Dictionary.condGet self.dict ID NotFound}
               of !NotFound then
                  raise unknownID(ID) end
               [] Val then
                  {self handleGet(ID Val $)}
               end
            end
         end
         meth condGet(ID Default $)
            lock
               %%{App.show condGet(ID Default)}
               case {Dictionary.condGet self.dict ID NotFound}
               of !NotFound then
                  %%{App.show notFound}
                  Default
               [] Val then
                  %%{App.show found}
                  {self handleGet(ID Val $)}
               end
            end
         end
         meth handleGet(ID Val $)
            Val
         end
         %%
         meth remove(ID)
            lock
               %%{App.show remove(ID)}
               case {Dictionary.condGet self.dict ID NotFound}
               of !NotFound then skip
               [] V then
                  {Dictionary.remove self.dict ID}
                  %%{App.show handleRemove(ID V)}
                  {self handleRemove(ID V)}
               end
            end
         end
         meth handleRemove(ID Val)
            skip
         end
      end
      /*
      class AutoRemove_Registry
         from GenericRegistry
            
         meth handleGet(ID Val $)
            GenericRegistry, remove(ID)
            Val
         end
      end
      */
      class TmpFile_Registry
         from GenericRegistry

         meth register(TmpFile ?ID)
            GenericRegistry, put(TmpFile ?ID)
         end
         meth condGetContentAndRemove(ID Default ?ContentVS)
            GenericRegistry, condGet(ID Default ?ContentVS)
         end
         meth handleGet(ID TmpFile ?ContentBS)
            ContentBS = {File.loadBS TmpFile}
            GenericRegistry, remove(ID)
         end
         meth handleRemove(ID TmpFile)
            {OS.unlink TmpFile}
         end
      end
   in
      TmpFiles = {New TmpFile_Registry init}
   end

   %%
   %% Handler for method createILoui.
   %% Result is a URL where the applet can be downloaded using HTTP GET.
   %%
   proc {CreateILouiApplet PREFIX Args Params ?Result}
      case Params of [RemoteURL SessionID] then
         Tmp = {OS.tmpnam}
         local
            Props = spec('ILOUI_REMOTEURL': RemoteURL
                         'ILOUI_SESSIONID': SessionID
			 'LOUI_TICKET': Args.brokerticket
			 'LOUI_MODE': "emacs"
                        )
         in
            PropertyEnv = if {HasFeature Args mode} then
                             {AdjoinAt Props 'LOUI_MODE' Args.mode}
                          else Props end
         end
         ILOUI_ORIG_APPLET_URL = "x-ozlib://mathweb/iloui.oza"
         ID
      in
         {MakeApplet.generic ILOUI_ORIG_APPLET_URL PropertyEnv Tmp}
         {TmpFiles register(Tmp ?ID)}
         Result = {VS2BS PREFIX#"/getILouiApplet/"#ID}
         thread
            {Delay 60000}
            {TmpFiles remove(ID)}
         end
      else
         raise fault(3 'Error: Illegal number of params.') end
      end
   end
   
   %%
   %% the created applets can be downloaded via HTTP GET 
   %%
   class GetAppletHandler from HTTPHandler

      meth handle(UriInContext Request Connection)
	 case UriInContext.path
	 of [ID] andthen {String.isInt ID} then
	    AppletID = {StringToInt ID} in
	    case {TmpFiles condGetContentAndRemove(AppletID false $)}
	    of false    then {Connection errNotFound}
	    [] AppletVS then {Connection applet(AppletVS)}
	    end
	 else
	    {Connection errNotFound}
	 end
      end
   end
 
end


