%%% -*- indent-tabs-mode: nil -*-
%%%
%%% Author:
%%%   Juergen Zimmer <jzimmer@mathweb.org>
%%%
%%% Copyright:
%%%   Juergen Zimmer, 2001
%%%
%%% Last change:
%%%   $Date: 2001/09/18 11:27:12 $ by $Author: jzimmer $
%%%   $Revision: 1.4 $
%%%
%%% This file is part of MathWeb
%%%    http://www.mathweb.org/mathweb
%%%
%%% See the file "LICENSE" or
%%%    http://www.mathweb.org/mathweb/LICENSE.html
%%% for information on usage and redistribution
%%% of this file, and for a DISCLAIMER OF ALL
%%% WARRANTIES.
%%%

/**
% This is a service registry for alive service objects of the xmlrpcbroker
*/

functor

import
   OS(rand srand time)
   Browser(browse: Browse)
   
export
   'class': ServiceRegistry
   
define
   
   VS2A = VirtualString.toAtom
   VS2S = VirtualString.toString

   {OS.srand 0}
   
   ServiceTimeout =  300      %% 300 seconds == 5 minutes
   
   class ServiceRegistry
      prop
         locking
      feat
         broker
         dict
         times
         
      meth init(broker:Broker)
         self.broker  = Broker
         self.dict    = {NewDictionary}
         self.times   = {NewDictionary}
      end
      
      meth NewID(ServiceName $)
         NewName =  {VS2A ServiceName#{OS.rand}}
      in
         if {Dictionary.member self.dict NewName} then
            {self NewID(ServiceName $)}
         else
            NewName
         end
      end
      
      meth new(ServiceName ?ID)
         lock
            Done 
            SObject 
         in
            {self.broker request(ServiceName ?SObject done: ?Done)}
            case Done 
            of unit then
               case SObject
               of false then
                  raise unknownService(ServiceName) end
               [] Obj then
                  {Obj enter}
                  ID = {self NewID(ServiceName $)}
                  {Dictionary.put self.dict ID Obj}
                  {Dictionary.put self.times ID {OS.time}}
               end
            [] system(os(...)...) then
               raise serverCrash(ServiceName) end
            else
               raise unknownService(ServiceName) end
            end
         end
         
         proc{WaitTimeout}
            case {Dictionary.condGet self.times ID unit}
            of unit then skip
            [] T then
%               {Browse ({OS.time} - T)}               
               if ({OS.time} - T) =< ServiceTimeout then
                  {WaitTimeout}
               end
            end
         end
      in
         thread
            {Thread.setPriority {Thread.this} low}
            {WaitTimeout}
            thread
               try
                  {self release(ID)}
               catch _ then skip end
            end
         end
      end
      
      meth send(ID Msg) = M
         case {Dictionary.condGet self.dict ID false}
         of false then
            raise unknownID(ID) end
         [] Obj then
            {Obj Msg}
            thread
               {Dictionary.put self.times ID {OS.time}}
            end
         end
      end
      
      meth release(ID)
         lock
            case {Dictionary.condGet self.dict ID false}
            of false then
               raise unknownID(ID) end
            [] Obj then
               try {Obj leave}
               catch _ then skip end
               {Dictionary.remove self.dict ID}
               {Dictionary.remove self.times ID}
            end
         end
      end
      
      meth releaseAll
         lock
            {ForAll {Dictionary.items self.dict}
             proc {$ Obj}
                try {Obj leave}
                catch _ then skip end
             end}
            {Dictionary.removeAll self.dict}
         end
      end
   end
   
end


