%%% -*- indent-tabs-mode: nil -*-
%%%
%%% Author:
%%%   Andreas Franke <afranke@mathweb.org>
%%%
%%% Copyright:
%%%   Andreas Franke, 2000-2001
%%%
%%% Last change:
%%%   $Date: 2001/10/24 06:53:09 $ by $Author: afranke $
%%%   $Revision: 1.2 $
%%%
%%% This file is part of MathWeb
%%%    http://www.mathweb.org/mathweb
%%%
%%% See the file "LICENSE" or
%%%    http://www.mathweb.org/mathweb/LICENSE.html
%%% for information on usage and redistribution
%%% of this file, and for a DISCLAIMER OF ALL
%%% WARRANTIES.
%%%

/**
% This is a web demo of an XML-RPC client.
*/

functor

import
   Error
   Compiler
   Browser(browse: Browse)
   
   XApplication
   at 'x-ozlib://mathweb/share/XApplication.ozf' 

   File(loadVS)
   at 'x-ozlib://mathweb/share/File.ozf'

   BrokerAccess(getBroker)
   at 'x-ozlib://mathweb/sys/BrokerAccess.ozf'

   HttpHandler('class':BaseHTTPHandler)
   at 'x-ozlib://mathweb/httpd/HttpHandler.ozf'
   
   XmlRpcClient
   at 'x-ozlib://mathweb/xmlrpc/XmlRpcClient.ozf'
   
   XmlRpcWriter
   at 'x-ozlib://mathweb/xmlrpc/xmlrpc.printer.ozf'

   Buffered('class':Buffer)
   at 'x-ozlib://mathweb/html/Buffered.ozf'
   
   HTML('class':  Html
        escape:   Esc
        /* escapeJS: EscapeJS */)
   at 'x-ozlib://mathweb/html/Html.ozf'

   XOpen(wrapper:SocketEmulator)
   at 'x-ozlib://mathweb/share/XOpen.ozf'
   
define
   
   %%VS2A = VirtualString.toAtom
   VS2S = VirtualString.toString
   VS2BS = ByteString.make
   
   fun {V2VS X} {Value.toVirtualString X 1000 1000} end
   
   AppName = 'rpcdemo'
   App     = {New XApplication.manager init}
   
   Spec = record
   Args
   try
      Args = {App.getCmdArgs Spec}
   catch error(ap(usage S)...) then
      {App.showError AppName#": "#S}
      {App.exit 1}
   end

   Broker = {BrokerAccess.getBroker App Args}.send
   
   HttpServer = {Broker request('HTTP_SERVER' locally: true  $)}
   if HttpServer == false then
      {App.showError "HTTP_SERVER not found."}
      {App.exit 1}
   end

   HttpHost = {HttpServer getHost($)}
   HttpPort = {HttpServer getPort($)}
   
   proc {DebugBrowse X}
      if Args.debug then {Browser.browse X} end
   end
 
   fun {ParseQuery QueryString Spec}
      {XApplication.parseCgiArgs Spec QueryString}
   end

   fun {EscVS VS}
      {VS2BS {Esc {VS2S VS}}}
   end
   local
      fun {EscQuote S}
         case S of nil then &"|nil
         [] C|R then
            case C of &" then &\\|&"|{EscQuote R}
            else C|{EscQuote R} end
         end
      end
   in
      fun {QuoteString S}
         &"|{EscQuote {Esc S}}
      end
   end
   fun {TT Val}
      tt({EscVS Val})
   end
      
   local
      class Buf from Buffer XmlRpcWriter.'class'
         meth write(vs:VS)
            Buffer, append(VS)
         end
      end
   in
      fun {MethodCallToVS MethodName Params}
         B = {New Buf reset} in
         {B 'write_methodCall'(MethodName Params)}
         {B getAll($)}
      end
   end
   
   class ConnectionManager from SocketEmulator Html 
      feat conn
      meth setConnection(Connection)
         self.conn = Connection
         SocketEmulator, init(Connection) 
      end
      meth otherwise(Msg) {self.conn Msg} end
   end

   class Self=HTTPHandler from BaseHTTPHandler
                             
      meth init() skip end
      
      meth handle(UriInContext Request Connection)
         Conn = {New ConnectionManager setConnection(Connection)} in
         case {Request getMethodS($)}
         of "GET" then
            Self, handleGET(UriInContext Request Conn)
         [] Method then
            {Connection errMethodNotImplemented(Method {Request getUriS($)})}
         end
      end

      meth handleGET(UriInContext Request Connection)
         case {CondSelect UriInContext path nil}
         of [nil] then
            Self, writeForm(Connection)
         [] ["rpc"] then
            Self, rpc(UriInContext.query Connection)
         else
            {Connection errNotFound}
         end
      end

      meth writeForm(Connection)
         /*
         FormFile = "/home/afranke/import/mathweb/xmlrpc/src/mathweb/form.html"
         FormVS   = {File.loadVS FormFile}
      in
         {Connection ok(FormVS)}
         */
         Title = "XMLRPC Demo"
         Content =
         form(action:"rpc" method:"get"
              "Host: " input(type:text size:25 name:"host" value:HttpHost)
              "&nbsp;&nbsp;"
              "Port: " input(type:text size:6 name:"port" value:HttpPort)
              "&nbsp;&nbsp;"
              "Handler: " input(type:text size:10 name:"handler" value:"mbase")
              br
              "Send handler as part of: "
              input(type:radio name:"rpc2" value:"on" checked:"") "MethodName"
              input(type:radio name:"rpc2" value:"off") "URL path"
              /*
              "Java-style handler-as-part-of-methodname:" 
              input(type:checkbox name:rpc2 checked:'')
              */
              br
              "MethodName: "
              input(type:text name:"method" value:"getRawItemByID")
              br
              "Params: " input(type:text name:"param" value:"1022")
              br 
              input(type:submit value:"Submit")
             )
         Document = html(head(title(Title))
                         body(bgcolor:"#ffffff" h1(Title) Content)) 
      in
         {Connection beginResponse}
         {Connection tag(Document)}
         {Connection endResponse}
      end

      meth rpc(Query Connection)
         Spec = record(host(single char:&h type:string default:"localhost")
                       port(single char:&p type:int    default:8080)
                       rpc2(single type:string default:"off") % checkbox
                       handler(single char:&H type:string default:"Broker")
                       methodName(single char:&M type:string default:"ping")
                       params(multiple char:&P type:list(string) default:nil)
                      )
         Args = {ParseQuery Query Spec}
         proc {ShowInfo VS}
            {Connection tag({EscVS VS}#br#"\n")}
         end
         proc {ShowPRE VS}
            {Connection tag(table(border:1
                                  tr(td(tt("\n"#{EscVS VS}#"\n"))))#br#"\n")}
         end
         fun {IsAtomOrInt F}
            {IsAtom F} orelse {IsInt F}
         end
         fun {ValToHtml X}
            if X==unit then
               "nil"
            elseif {IsBool X} then
               if X then "1" else "0" end
            elseif {IsInt X} then
               {IntToString X}
            elseif {IsFloat X} then
               {FloatToString X}
            elseif {IsTuple X} andthen {Label X}=='#' then
               table(border:1 cellpadding:3
                     tr(th(colspan:{Width X} i("Array")))
                     tr({Record.map X fun {$ V} td({ValToHtml V}) end}))
            elseif {IsString X} then
               {QuoteString X}
            elseif {IsList X} then
               table(tr(th(colspan:{Length X} i("Array")))
                     tr({List.toTuple '#' fun {$ V} td({ValToHtml V}) end}))
            elseif {IsAtom X} then
               {QuoteString X}
            elseif {IsRecord X} andthen
               ({IsAtom {Label X}} orelse {IsUnit {Label X}}) andthen
               {All {Arity X} IsAtomOrInt}
            then
               table(border:1 cellpadding:3
                     tr(th(colspan:2 align:left
                           if {IsUnit {Label X}} then i("Record")
                           else b({Esc {V2VS {Label X}}}) end))
                     {List.toTuple '#'
                      {Map {Record.toListInd X}
                       fun {$ F#V} tr(td({Esc {AtomToString F}})
                                      td({ValToHtml V})) end}})
            else
               {QuoteString X}
            end
         end
         
         Url = "http://"#Args.host#":"#Args.port#
         if Args.rpc2=="on" then "/RPC2" else "/RPC2/"#Args.handler end
         
         MethodName =
         if Args.rpc2=="on" then Args.handler#"." else "" end # Args.methodName
        
         Params = {Map Args.params Compiler.virtualStringToValue}

         {Connection beginResponse}
         {Connection beginContentStream}
         {Connection startTag(html)}
         {Connection tag(head(title("XMLRPC Result")))}
         {Connection startTag(body(bgcolor:"#ffffff"))}
         {Connection tag(h1("XMLRPC Result"))}
         
         {Connection tag(table(border:1 cellpadding:3
                               tr(td("POSTing to URL: ") td({TT Url}))
                               tr(td("MethodName: ") td({TT MethodName}))
                               tr(td("Params: ")
                                  td(table({List.toTuple '#'
                                            {Map Params
                                             fun {$ P}
                                                tr(td({TT {V2VS P}}))
                                             end}}))))#p)}
         
         {ShowInfo "Request body:"}
         {ShowPRE {MethodCallToVS MethodName Params}}

         %% begin XMLRPC client call
         ?Result ?Response
         C = {New XmlRpcClient.'class' init(Url)}
         try
            try
               {C execute(MethodName Params ?Result
                          response: ?Response)}
            finally
               {ShowInfo "Response body:"}
               {ShowPRE Response.content}
               {ShowInfo "Interpreted result: "}
            end
            {Connection tag({ValToHtml Result})}
         catch Err then
            E={Record.subtract Err debug}
            VS={Error.messageToVirtualString {Error.exceptionToMessage E}}
         in
            {Connection tag(pre({Esc VS}))}
         finally
            {C close}
         end
         %% end XMLRPC client call
      in
         {Connection endTag(body)}
         {Connection endTag(html)}
         {Connection endResponse}
      end
   end

   HandlerObj = {New HTTPHandler init}
   
   %% register
   {HttpServer addHandler("/rpcdemo" HandlerObj)}
   
   {App.exit 0}
end

