%%% -*- indent-tabs-mode: nil -*-
%%%
%%% Author:
%%%   Andreas Franke <afranke@mathweb.org>
%%%
%%% Copyright:
%%%   Andreas Franke, 2000-2001
%%%
%%% Last change:
%%%   $Date: 2001/09/18 11:27:12 $ by $Author: jzimmer $
%%%   $Revision: 1.7 $
%%%
%%% This file is part of MathWeb
%%%    http://www.mathweb.org/mathweb
%%%
%%% See the file "LICENSE" or
%%%    http://www.mathweb.org/mathweb/LICENSE.html
%%% for information on usage and redistribution
%%% of this file, and for a DISCLAIMER OF ALL
%%% WARRANTIES.
%%%

/**
% This is a server that handles XML-remote-procedure-calls and
% forwards them to the MathWeb meta-services.
*/

functor

import
   URL
   Browser(browse: Browse)
   
   XApplication
   at 'x-ozlib://mathweb/share/XApplication.ozf' 

   BrokerAccess(getBroker)
   at 'x-ozlib://mathweb/sys/BrokerAccess.ozf'

   XService(metaService: MetaService
            server: Server )
   at 'x-ozlib://mathweb/sys/XService.ozf'

   XmlRpcHandler('class':RpcHandler)
   at 'x-ozlib://mathweb/xmlrpc/XmlRpcHandler.ozf'

   XmlService('class': XmlRpcService)
   at 'x-ozlib://mathweb/sys/XmlRpcService.ozf'
   

   ServiceRegistry('class': ServRegistry)
   at 'ServiceRegistry.ozf'
   
   Passwds(passwords: Passwords)
   at '/home/mathweb/Passwd.ozf'
   
define
   VS2A = VirtualString.toAtom
   VS2S = VirtualString.toString
   
   AppName = 'objectbroker'
   App     = {New XApplication.manager init}
   
   Spec = record(brokerticket(single char:[&t &b] type:string           optional:true)
                 debug(       single char:&d      type:bool             default:false)
                 loglevel(    single char:&l      type:int(min:0 max:9) default:1)
                )
   Args
   try
      Args = {App.getCmdArgs Spec}
   catch error(ap(usage S)...) then
      {App.showError AppName#": "#S}
      {App.exit 1}
   end
   
   proc {DebugBrowse X}
      if Args.debug then {Browser.browse X} end
   end
   
   Broker = {BrokerAccess.getBroker App Args}.send
   
   RpcServer = {Broker request('XMLRPC_SERVER' locally: true  $)}
   if RpcServer == false then
      {App.showError "XMLRPC_SERVER not found."}
      {App.exit 1}
   end
   
   PREFIX = case {Reverse {VS2S {RpcServer getBaseURL($)}}}
            of &/|ReverseBaseUrl then {Reverse ReverseBaseUrl}
            [] ReverseBaseUrl then {ReverseBaseUrl}
            end

   %% ==============================================
   %% The ObjectBroker handles 'getService' and 'offerService' requests
   %% ==============================================
   class Self=ObjectBroker from RpcHandler

      attr
         xmlrpcServer: unit
         registry:     unit % maintains the different service objects
         
      meth init(XMLRPCServer)
         xmlrpcServer <- XMLRPCServer
         registry     <- {New ServRegistry init(broker: Broker)}
      end
      
      meth cleanup
         {@registry releaseAll}
      end
      
      %%
      %% this method is called to handle XMLRPC requests
      %%
      meth execute(MethodNameVS Params ?Result) 
         case {VS2S MethodNameVS}
         of "getService" then
            Self, exec_getService(Params ?Result)
         [] "offerService" then
            Self, exec_offerService(Params ?Result)
         [] "retireService" then
            Self, exec_retireService(Params ?Result)
         else
            RpcHandler, execute(MethodNameVS Params ?Result)
         end
      end
      
      %%
      %% and here are methods for each <methodName>
      %%
      meth exec_getService(Params ?Result)
         case Params
         of nil then
            raise fault(41 "Error: Please give a service name and your password.") end
%         [] [Name Passwd] then
         [] [Name] then
            try 
%                if {Not {Member Passwd Passwords}} then    %% expects a ByteString!
%                   raise  wrongPassword(Passwd) end
%                end
               ServiceName = {VS2A {Map {VS2S Name} Char.toUpper}} 
               ObjectID = {@registry new(ServiceName $)}
               ObjectHandler =
               %% create a new handler for the service object
               
               proc {$ MethodS Pars ?Res}
                  Method = {VS2A MethodS}
               in
                  case Method
                  of kill then
                     try
                        {@registry release(ObjectID)}
                        {@xmlrpcServer removeHandler(ObjectID)}
                        Res = true
                     catch _ then
                        Res = true
                     end
                  else
                     case Pars of [Par1] then
                        if {IsRecord Par1} then 
                           ResultPos = {FoldR {Filter {Arity Par1} IsInt} Max 0} + 1
                           MethodCall = {Adjoin Par1 Method(ResultPos: ?Res)}
                        in
%                        {Browse pos(ResultPos)}
%                        {Browse call(MethodCall)}
                           {@registry send(ObjectID MethodCall)}
                        else
                           {@registry send(ObjectID Method(Par1 ?Res))}
                        end
                     else {@registry send(ObjectID Method(Pars ?Res))}
                     end
                  end
               end
            in
               {DebugBrowse newID(ObjectID)}
               {@xmlrpcServer addHandler(ObjectID ObjectHandler mode:fun2)}
               Result = {VS2S PREFIX#"/RPC2/"#ObjectID}
            catch unknownService(Name) then
               raise fault(3 {VS2S "Error: Service "#Name#" is unknown."}) end
            [] serverCrash(Name) then
               raise fault(5 {VS2S "Error: The service server for "#Name#" seens to have crashed."}) end
            [] wrongPassword(Passwd) then
               raise fault(4 {VS2S "Error: Invalid password: "#Passwd}) end
            [] E then
               {Browse E}
            end
         else
            raise fault(41 "Error: Please give a service name and your password.") end
         end   
      end

      meth exec_offerService(Params ?Result)
         case Params
         of [Name ServerUrl] then
            try
%                if {Not {Member Passwd Passwords}} then    %% expects a ByteString!
%                   raise  wrongPassword(Passwd) end
%                end
               ServiceName = {VS2A {Map {VS2S Name} Char.toUpper}} 
%          %%----------------------------------------------
%          %%  methods for handling of (local) serivces
%          %%----------------------------------------------
%          meth offer(ServiceName Service ?ServiceID
%                     mode:      Mode       <= create
%                     host:      Host       <= "<unknown>"
%                     signature: Signature  <= unit
%                     done:      Done       <= _
%                    )
               UrlRec = {URL.make ServerUrl}
               {Browse offering(Name ServerUrl)}
               Done ServiceID Server
               proc {DoExit _}
                  try {Broker retire(ServiceID)}
                  catch _ then skip end
               end
            in
               
               Server = {App newServer(MetaService
                                       init(name:     ServiceName
                                            'class':  XmlRpcService
                                            url:      ServerUrl
                                            ticket:   Name
                                            appMan:   App
                                           )
                                       $
                                       detach: true
                                       info:   ServiceName#'.xmlrpc-service'
                                       onExit: DoExit)}
               %%{App.showInfo ' done.'}
               {App.printInfo "Offering service `"#ServiceName#"'..."}
               {Broker offer(ServiceName
                             Server
                             ?ServiceID
                             host: {CondSelect UrlRec authority "<unknown>"}
                            )}
               {App.showInfo " done."}
               Result = ServiceID
               
            catch _ then
               raise fault(100 {VS2S "Error: Some error while offering service."}) end
            end
         else
            raise fault(41 "Error: Please give a service name and the URL of the XML-RPC meta-service.") end
         end   
      end
      
      meth exec_retireService(Params ?Result)
         case Params
         of [ServiceID] then
            try
               {Broker retire(ServiceID)}
               {App.showInfo " done."}
               Result = "done"
            catch _ then
               raise fault(100 {VS2S "Error: Some error while offering service."}) end
            end
         else
            raise fault(41 "Error:  Please give a service ID.") end
         end   
      end
   end
   
   ObjBroker = {New ObjectBroker init(RpcServer)}
   
   %% register
   {RpcServer addHandler("Broker" ObjBroker
                         onRemove: proc {$} {ObjBroker cleanup} end)}
   
   {App.exit 0}
end

