%%% -*- indent-tabs-mode: nil -*-
%%%
%%% Author:
%%%   Juergen Zimmer <jzimmer@mathweb.org>
%%%   Andreas Franke <afranke@mathweb.org>
%%%
%%% Copyright:
%%%   Juergen Zimmer, 2000-2001
%%%
%%% Last change:
%%%   $Date: 2001/09/14 10:38:10 $ by $Author: jzimmer $
%%%   $Revision: 1.6 $
%%%
%%% This file is part of MathWeb
%%%    http://www.mathweb.org/mathweb
%%%
%%% See the file "LICENSE" or
%%%    http://www.mathweb.org/mathweb/LICENSE.html
%%% for information on usage and redistribution
%%% of this file, and for a DISCLAIMER OF ALL
%%% WARRANTIES.
%%%

/**
% This is a generic xmlrpc server. It can be run in a mathweb environment
% where "broker" and "httpd" are already running.
*/

functor

import
   XApplication
   at 'x-ozlib://mathweb/share/XApplication.ozf'

   BrokerAccess(getBroker)
   at 'x-ozlib://mathweb/sys/BrokerAccess.ozf'

   XmlRpcServer(httpHandler)
   at 'x-ozlib://mathweb/xmlrpc/XmlRpcServer.ozf'
   
   DummyHandler(call)
   at 'x-ozlib://mathweb/xmlrpc/DummyHandler.ozf'
   
define
   AppName = 'xmlrpcd'
   App     = {New XApplication.manager init}
   
   Spec = record(brokerticket(single char:[&t &b] type:string optional:true)
		 debug(single char:&d type:bool default:false)
		)
   Args
   try
      Args = {App.getCmdArgs Spec}
   catch error(ap(usage S)...) then
      {App.showError AppName#": "#S}
      {App.exit 1}
   end

   Broker = {BrokerAccess.getBroker App Args}.send
   
   HTTPServer = {Broker request('HTTP_SERVER' locally: true $)}
   if HTTPServer == false then
      {App.showError "HTTP_SERVER not found."}
      {App.exit 1}
   end
   
   RpcServer = {New XmlRpcServer.httpHandler init}

   XmlrpcServerThreadID
   proc {OnHttpHandlerRemove}
      {{App parent($)} tryTerminate(XmlrpcServerThreadID)}
   end
   HttpHandlerID = {HTTPServer addHandler("/RPC2" RpcServer $
                                          onRemove: OnHttpHandlerRemove)}
   {App.showInfo "HTTP handler id = "#HttpHandlerID}
   
   {RpcServer addHandler("/" DummyHandler.call mode:fun2)}

   ServiceID
   
   class XmlrpcService

      meth init(...) skip end
      meth enter(...) skip end
      meth leave(...) skip end

      meth ping($) unit end

      meth getHost(...)=Msg    {HTTPServer Msg} end
      meth getPort(...)=Msg    {HTTPServer Msg} end
      meth getBaseURL(...)=Msg {HTTPServer Msg} end

      meth addHttpHandler(...)=Msg
         {HTTPServer {Adjoin Msg addHandler}}
      end
      meth removeHttpHandler(...)=Msg
         {HTTPServer {Adjoin Msg removeHandler}}
      end
      
      meth addHandler(...)=Msg    {RpcServer Msg} end
      meth removeHandler(...)=Msg {RpcServer Msg} end
      meth showHandlers(...)=Msg  {RpcServer Msg} end
      
      meth 'SHUTDOWN'
         {App.showInfo "*** xmlrpcd shutdown"}
         try
            %%{App.printInfo "*** unregistering from broker..."}
            {WaitOr {Alarm 1000}
             {Broker retire(ServiceID done:$)}}
            %%{App.showInfo " done"}
         catch Err then
            {App printExc(Err)}
         end
         try
            {App.printInfo "*** unregistering from http server..."}
            {HTTPServer removeHandler(HttpHandlerID)}
            {App.showInfo " done"}
         catch Err then
            {App printExc(Err)}
         end
         try
            {App.printInfo "*** removing all handlers..."}
            {RpcServer removeAllHandlers}
            {App.showInfo " done"}
         catch Err then
            {App printExc(Err)}
         end
      end
   end

   XmlrpcServer = {App newServer(XmlrpcService init $ ?XmlrpcServerThreadID
                                 detach: true
                                 info:   "xmlrpcd")}
   
   %% register xmlrpc server at the broker
   {Broker offer('XMLRPC_SERVER' XmlrpcServer ?ServiceID mode:pass_on)}
   
   {App.exit 0}
end





