%%% -*- indent-tabs-mode: nil -*-
%%%
%%% Author:
%%%   Juergen Zimmer <jzimmer@mathweb.org>
%%%   Andreas Franke <afranke@mathweb.org>
%%%
%%% Copyright:
%%%   Juergen Zimmer, 2001
%%%
%%% Last change:
%%%   $Date: 2001/09/13 15:39:13 $ by $Author: jzimmer $
%%%   $Revision: 1.9 $
%%%
%%% This file is part of MathWeb
%%%    http://www.mathweb.org/mathweb
%%%
%%% See the file "LICENSE" or
%%%    http://www.mathweb.org/mathweb/LICENSE.html
%%% for information on usage and redistribution
%%% of this file, and for a DISCLAIMER OF ALL
%%% WARRANTIES.
%%%

%%
%% a printer for xmlrpc values
%%

functor
   
import
   Type(ask)
%   Browser(browse:Browse)   
   
   Base64
   at 'x-ozlib://mathweb/share/Base64.ozf'
   
   XmlPrinter('class': XML)
   at 'x-ozlib://mathweb/share/XmlPrinter.ozf'
   
   %% PP('class': PrettyPrinter)
   %% at 'x-ozlib://mathweb/html/PrettyPrinter.ozf'

export
   'class': ValueWriter
   
define

   VS2S = VirtualString.toString
   
   fun {EscapeXml S}
      case S of nil then nil
      [] C|R then
	 case C
	 of &<  then &&|&#|&6|&0|&;|{EscapeXml R}
	 [] &>  then &&|&#|&6|&2|&;|{EscapeXml R}
	 [] &&  then &&|&#|&3|&8|&;|{EscapeXml R}
	 [] &\\ then &&|&#|&9|&2|&;|{EscapeXml R}
	 else C|{EscapeXml R}
	 end
      end
   end
   
   %%
   %% write xmlrpc values
   %%
   local
      OZ_LABEL = '__oz_label'

      fun {IsAtomOrInt F}
         {IsAtom F} orelse {IsInt F}
      end
      TypeAskAtomOrInt = {Type.ask.generic IsAtomOrInt 'atom_or_int'}
   in
      class Self = ValueWriter
         from XML /*PrettyPrinter*/
                                  
	 meth 'write_value'(X)
	    %% base64 = base64(VS)
            %% array  = tuple with label '#'
            %% struct = record, all features are atoms
            %% nil    = unit (empty atom)
            {self startTag(value)}
            %%{Browse X}
            case X of unit then
	       Self, 'write_nil'()
	    [] base64(VS) then
               Self, 'write_base64'(VS)
	    elseif {IsBool X} then
	       Self, 'write_boolean'(X)
	    elseif {IsInt X} then
	       Self, 'write_int'(X)
            elseif {IsFloat X} then
               Self, 'write_double'(X)
            elseif {IsTuple X} andthen {Label X}=='#' then
	       Self, 'write_array'(X)
	    elseif {IsString X} then
	       Self, 'write_string'(X)
	    elseif {IsList X} then
	       Self, 'write_array'({List.toTuple '#' X})
            elseif {IsAtom X} then
               Self, 'write_string'({VS2S X})
	    elseif {IsRecord X} andthen
	       ({IsAtom {Label X}} orelse {IsUnit {Label X}}) andthen
	       {All {Arity X} IsAtomOrInt}
	    then
               Self, 'write_struct'(X)
	    else
               Self, 'write_string'(X)
            end
            {self endTag(value)}
         end
         
         %%
         %% scalar values
         %%
         
	 meth 'write_boolean'(X)
            {Type.ask.bool X}
            {self startTag(boolean)}
            if X
            then {self write(vs:"1")}
            else {self write(vs:"0")}
            end
            {self endTag(boolean)}
	 end
         meth 'write_int'(X)
            {Type.ask.int X}
            {self startTag(int)}
            {self write(vs:{IntToString X})}
            {self endTag(int)}
	 end
         meth 'write_double'(X) 
            {Type.ask.float X}
            {self startTag(double)}
            {self write(vs:X)}
            {self endTag(double)}
         end
         meth 'write_string'(X)
            {Type.ask.virtualString X}
            {self startTag(string)}
	    {self write(vs:{EscapeXml {VS2S X}})}
            {self endTag(string)}
	 end
         meth 'write_base64'(X)
            {Type.ask.virtualString X}
            {self startTag(base64)}
            {self write(vs:{Base64.encode X})}
            {self endTag(base64)}
         end
         meth 'write_dateTime.iso8601'(X)
            raise unimplemented('dateTime.iso8601' X) end
         end
         meth 'write_nil'()
            {self emptyTag(nil)}
         end
         
         %%
         %% <struct>s 
         %%
         meth 'write_struct'(X)
            {Type.ask.record X}
            {self startTag(struct)}
            case {Label X} of unit then skip
            [] L then
               Self, 'write_member'(OZ_LABEL {AtomToString L})
            end
            {Record.forAllInd X
             proc {$ F V}
                Self, 'write_member'(F V)
             end}
            {self endTag(struct)}
         end
         meth 'write_member'(F V)
            {self startTag(member)}
            Self, 'write_name'(F)
            Self, 'write_value'(V)
            {self endTag(member)}
         end
         meth 'write_name'(X)
            {TypeAskAtomOrInt X}
            {self startTag(name)}
            {self write(vs:X)}
            {self endTag(name)}
         end

         %%
         %% <array>s
         %%
         meth 'write_array'(X)
            {self startTag(array)}
            Self, 'write_data'(X)
            {self endTag(array)}
         end
         meth 'write_data'(X)
            {Type.ask.tuple X}
            {self startTag(data)}
            {Record.forAll X
             proc {$ V}
                Self, 'write_value'(V)
             end}
            {self endTag(data)}
         end
      end
   end
end



