%%
%% parser semantics for xml-rpc values
%%

functor

import
   Base64 % at 'x-oz://contrib/base64.so{native}'
   at 'x-ozlib://mathweb/share/Base64.ozf'

   Type(ask)
   
export
   'class': SemValue
   
define

   VS2A  = VirtualString.toAtom
   VS2S  = VirtualString.toString
   VS2BS = VirtualString.toByteString
   
   fun {DropSpaces S} {List.dropWhile S Char.isSpace} end
   fun {Clear S} {Reverse {DropSpaces {Reverse {DropSpaces S}}}} end
   
   fun {ToBool S}
      case {Clear S}
      of "0" then false
      [] "1" then true
%     [] "false" then false
%     [] "true" then true
      else
         raise parser(type:bool value:{VS2BS S}) end
      end
   end
   
   fun {ToInt S}
      if {String.isInt S} then
         {StringToInt S}
      else
         raise parser(type:int value:{VS2BS S}) end
      end
   end
   
   fun {ToFloat S}
      if {String.isFloat S} then
         {StringToFloat S}
      else
         raise parser(type:float value:{VS2BS S}) end
      end
   end
   
   OZ_LABEL = '__oz_label'

   fun {IsTypedValue X}
      case X of typedValue(_)
      then true
      else false
      end
   end
   
   class SemValue

      meth 'make_value'(E $)
         case {Filter E.content IsTypedValue}
         of [typedValue(Val)] then Val
         [] nil then
            case E.content
            of nil then
               {ByteString.make ""}
            [] [VS] then
               {Type.ask.virtualString VS}
               {VS2BS VS}
            else
               raise 'internal error: unrecognized markup in a <value>' end
            end
         else
            raise 'multiple typed values found between <value>...</value>' end
         end
      end
      
      %%
      %% scalar values
      %%
      meth 'make_boolean'(E $)
         typedValue({ToBool {VS2S E.content}})
      end
      meth 'make_i4'(E $)
         typedValue({ToInt {VS2S E.content}})
      end
      meth 'make_int'(E $)
         typedValue({ToInt {VS2S E.content}})
      end
      meth 'make_double'(E $)
         typedValue({ToFloat {VS2S E.content}})
      end
      meth 'make_string'(E $)
         typedValue({VS2BS E.content})
      end      
      meth 'make_base64'(E $)
         typedValue({VS2BS {Base64.decode E.content}})
      end
      meth 'make_dateTime.iso8601'(E $)
         typedValue({VS2BS E.content})    %%raise unimplemented('dateTime.iso8601' E.content) end
      end
      meth 'make_nil'(E $)
         typedValue(unit)
      end
      
      %%
      %% <struct>s --> Records
      %%
      meth 'make_struct'(E $)
         case E.content
         of (!OZ_LABEL#L)|XYs then
            typedValue({List.toRecord {VS2A L} XYs})
         [] XYs then
            typedValue({List.toRecord unit XYs}) 
         end
      end
      meth 'make_member'(E $)
         E.content % = Name # Val
      end
      meth 'make_name'(E $)
         S = {VS2S E.content} in
         if {String.isInt S}
         then {StringToInt S} 
         else {StringToAtom S}
         end
      end
      
      %%
      %% <array>s --> Tuples
      %%
      meth 'make_array'(E $)
         typedValue(E.content)                  %   changed jz: 27.8.2001
%         typedValue({List.toTuple '#' E.content}) 
      end
      meth 'make_data'(E $)
         E.content
      end
   end
end










